import {
  Context,
  NotebookCellContext,
  NotebookContext,
  DocumentContext,
  GitRepoContext
} from './types/context';
import { CellData } from '../utils/extractCellData';
import { NotebookController } from '../controller';
import { getJovyanClient } from '../jovyanClient';

// --- Controller Class (Logic) ---
// Export the class so it can be imported by the provider
export class ChatContextController {
  private notebookController: NotebookController;

  constructor(notebookController: NotebookController) {
    this.notebookController = notebookController;
  }

  // Modifier logic: Takes current state, returns new state
  addContext(currentContexts: Context[], contextToAdd: Context): Context[] {
    if (
      !currentContexts.some(existingCtx => existingCtx.id === contextToAdd.id)
    ) {
      return [...currentContexts, contextToAdd];
    }
    return currentContexts; // Return unchanged state if already exists
  }

  // Modifier logic: Takes current state, returns new state
  addCurrentCellContext(currentContexts: Context[]): Context[] {
    const notebookPath = this.notebookController.getCurrentNotebookFilePath();
    const activeCell = this.notebookController.activeCell;
    if (activeCell) {
      const cellContext = new NotebookCellContext(
        activeCell.model.id,
        notebookPath
      );
      if (
        !currentContexts.some(
          ctx => ctx.id === cellContext.id && ctx.type === 'notebook-cell'
        )
      ) {
        console.debug('Controller adding current cell context', cellContext.id);
        return [...currentContexts, cellContext];
      } else {
        console.debug(
          'Controller: Current cell context already exists',
          cellContext.id
        );
      }
    } else {
      console.warn('Controller: No active cell found to add context for.');
    }
    return currentContexts; // Return unchanged state if no cell or already exists
  }

  addCurrentNotebookContext(currentContexts: Context[]): Context[] {
    const notebookPath = this.notebookController.getCurrentNotebookFilePath();
    if (notebookPath) {
      const notebookContext = new NotebookContext(notebookPath);
      return this.addContext(currentContexts, notebookContext);
    }
    return currentContexts;
  }

  // Modifier logic: Takes current state, returns new state
  removeContext(
    currentContexts: Context[],
    contextToRemove: Context
  ): Context[] {
    return currentContexts.filter(ctx => ctx.id !== contextToRemove.id);
  }

  // Read logic: Operates on current state
  hasCurrentCellInContext(currentContexts: Context[]): boolean {
    const activeCellId = this.notebookController.activeCell?.model.id;
    if (!activeCellId) {
      return false;
    }
    return currentContexts.some(
      ctx => ctx.id === activeCellId && ctx.type === 'notebook-cell'
    );
  }

  // Read logic: Doesn't depend on activeContexts state, only notebookController
  getContextName(context: Context): string {
    return this.notebookController
      ? context.getDisplayName(this.notebookController)
      : 'Context';
  }

  // Read logic: Operates on current state
  getContextDataForApi(
    currentContexts: Context[],
    currentNotebookInContext: boolean
  ): {
    currentNotebook: CellData[];
    selectedCells: CellData[];
    attachedContexts: Array<{ type: string; value: string[] }>;
  } {
    if (!this.notebookController) {
      console.error(
        'NotebookController not available in ChatContextController logic'
      );
      return { currentNotebook: [], selectedCells: [], attachedContexts: [] };
    }

    const currentNotebookContext = new NotebookContext(
      this.notebookController.getCurrentNotebookFilePath()
    );
    const currentNotebook = currentNotebookInContext
      ? currentNotebookContext.getContextDataForApi(this.notebookController)
      : [];
    const selectedCells = currentContexts
      .filter(ctx => ctx.type === 'notebook-cell')
      .map(ctx => ctx.getContextDataForApi(this.notebookController));

    const finalDocuments: Array<{ type: string; value: string[] }> = [];

    const docIds = currentContexts
      .filter(ctx => ctx.type === 'document')
      .map(ctx => ctx.id);
    if (docIds.length > 0) {
      finalDocuments.push({ type: 'document', value: docIds });
    }

    const repos = currentContexts
      .filter(ctx => ctx.type === 'git-repo')
      .map(ctx => ctx.id);
    if (repos.length > 0) {
      finalDocuments.push({ type: 'git-repo', value: repos });
    }

    return {
      currentNotebook: currentNotebook,
      selectedCells: selectedCells.filter(
        (cell): cell is CellData => cell !== null
      ),
      attachedContexts: finalDocuments
    };
  }

  async sendDocumentContext(
    name: string,
    file: File | null,
    link: string | null
  ): Promise<DocumentContext | void> {
    if (!file && !link) {
      console.warn('No file or link provided to sendDocumentContext.');
      return;
    }

    try {
      const client = await getJovyanClient();
      let documentName = name;
      if (!documentName) {
        if (file) {
          documentName = file.name.split('/').pop() || file.name;
        }
        if (link && !documentName) {
          try {
            const url = new URL(link);
            documentName = url.hostname;
          } catch (e) {
            documentName = 'linked_document';
          }
        }
      }

      const docContext = new DocumentContext(
        documentName ? documentName : file?.name || '',
        documentName,
        file || null,
        link || null
      );

      console.debug('Sending document context to backend:', {
        file: file?.name,
        link,
        documentName
      });

      await (client as any).sendDocumentToIndex(file, link, documentName);

      console.info('Document context sent successfully.');
      return docContext; // Return the context so UI can potentially use it
    } catch (error) {
      console.error('Failed to send document context:', error);
      throw error;
    }
  }

  async getIndexedDocumentNames(): Promise<string[]> {
    try {
      const client = await getJovyanClient();
      const documentNames = await (client as any).getIndexDocumentNames(
        'document'
      );
      return documentNames || [];
    } catch (error) {
      console.error('Failed to fetch indexed document names:', error);
      return [];
    }
  }

  async getIndexedRepoNames(): Promise<string[]> {
    try {
      const client = await getJovyanClient();
      const repoNames = await (client as any).getIndexDocumentNames('git-repo');
      return repoNames || [];
    } catch (error) {
      console.error('Failed to fetch indexed repository names:', error);
      return [];
    }
  }

  async sendGitRepoContext(
    name: string,
    url: string
  ): Promise<GitRepoContext | void> {
    try {
      const client = await getJovyanClient();
      const repoContext = new GitRepoContext(name, name, url);

      console.debug('Sending Git repo context to backend:', {
        url,
        name
      });

      await (client as any).sendDocumentToIndex(null, url, name);

      console.info('Git repo context sent successfully.');
      return repoContext;
    } catch (error) {
      console.error('Failed to send Git repo context:', error);
      throw error;
    }
  }
}
